#include <stdio.h>

typedef enum
{
    ADD,
    VAR,
} operator_t;

typedef struct exp_t
{
    operator_t op;
    union
    {
        struct exp_t *opnds[2];
        char const *var_name;
    } u;
} exp_t;

exp_t *exp_init_add(exp_t *t, exp_t *l, exp_t *r)
{
    t->op = ADD;
    t->u.opnds[0] = l;
    t->u.opnds[1] = r;
    return t;
}

exp_t *exp_init_var(exp_t *t, char const *n)
{
    t->op = VAR;
    t->u.var_name = n;
    return t;
}

void exp_print(exp_t const *t, FILE *f)
{
    switch (t->op)
    {
    case ADD:
        exp_print(t->u.opnds[0], f);
        fprintf(f, " + ");
        exp_print(t->u.opnds[1], f);
        break;
    case VAR:
        fputs(t->u.var_name, f);
        break;
    }
}

typedef struct env env_t;

int exp_eval(exp_t const *t, env_t const *e)
{
    switch (t->op)
    {
    case ADD:
        return exp_eval(t->u.opnds[0], e) + exp_eval(t->u.opnds[1], e);
    case VAR:
        return env_get(e, t->u.var_name);
    }
}

void *main()
{
    const char *a = "A";
    const char *b = "BD";
    exp_t v_a, v_b;
    exp_init_var(&v_a, a);
    exp_init_var(&v_b, b);

    exp_t add;
    exp_init_add(&add, &v_a, &v_b);

    exp_print(&add, stdout);
}