#include <stdio.h>
#include <stdlib.h>

typedef enum
{
    ADD,
    VAR,
} operator_t;

typedef struct env env_t;

int env_get(env_t const *e, char const *n)
{
    exit(0);
}

typedef struct
{
    int (*eval)(void const *this, env_t *e);
    void (*print)(void const *this, FILE *f);
} exp_vtab_t;

typedef struct exp_t
{
    exp_vtab_t *vtab;
    operator_t op;
    union
    {
        char const *var_name;
    } u;
} exp_t;

int exp_eval(exp_t const *t, env_t const *e)
{
    switch (t->op)
    {
    case VAR:
        return env_get(e, t->u.var_name);
    }
}

// exp_var
typedef exp_vtab_t exp_var_vtab_t;

typedef struct
{
    exp_var_vtab_t const *vtab;
    char const *var_name;
} exp_var_t;

int exp_var_eval(void const *t, env_t *e)
{
    exp_var_t const *this = t;
    return env_get(e, this->var_name);
}

void exp_var_print(void const *t, FILE *f)
{
    exp_var_t const *this = t;
    fputs(this->var_name, f);
}

static const exp_var_vtab_t exp_var_vtab = {
    exp_var_eval,
    exp_var_print,
};

exp_var_t *exp_init_var(exp_var_t *t, char const *n)
{
    t->vtab = &exp_var_vtab;
    t->var_name = n;
    return t;
}

// exp_add
typedef exp_vtab_t exp_add_vtab_t;

typedef struct
{
    exp_add_vtab_t const *vtab;
    exp_t *left;
    exp_t *right;
} exp_add_t;

int exp_add_eval(void const *t, env_t *e)
{
    exp_add_t const *this = t;
    exp_t *l = this->left;
    exp_t *r = this->right;
    int left = l->vtab->eval(l, e);
    int right = r->vtab->eval(r, e);
    return left + right;
}

void exp_add_print(void const *t, FILE *f)
{
    exp_add_t const *this = t;
    fprintf(f, "(");
    this->left->vtab->print(this->left, f);
    fprintf(f, " + ");
    this->right->vtab->print(this->right, f);
    fprintf(f, ")");
}

static const exp_add_vtab_t exp_add_vtab = {
    exp_add_eval,
    exp_add_print,
};

exp_add_t *exp_init_add(exp_add_t *t, exp_t *l, exp_t *r)
{
    t->vtab = &exp_add_vtab;
    t->left = l;
    t->right = r;
    return t;
}

// exp_const
typedef struct
{
    int (*eval)(void const *this, env_t *e);
    void (*print)(void const *this, FILE *f);
    int (*get_value)(void const *this);
} exp_const_vtab_t;

typedef struct
{
    exp_const_vtab_t const *vtab;
    int value;
} exp_const_t;

int exp_const_eval(void const *t, env_t *e)
{
    exp_const_t const *this = t;
    return this->value;
}

void exp_const_print(void const *t, FILE *f)
{
    exp_const_t const *this = t;
    fprintf(f, "%d", this->value);
}

int exp_const_get_value(void const *t)
{
    exp_const_t const *this = t;
    return this->value;
}

static const exp_const_vtab_t exp_const_vtab = {
    exp_const_eval,
    exp_const_print,
    exp_const_get_value};

exp_const_t *exp_init_const(exp_const_t *t, int value)
{
    t->vtab = &exp_const_vtab;
    t->value = value;
    return t;
}

// main
void *main()
{
    const char *a = "A";
    const char *b = "B";
    exp_var_t v_a, v_b;
    exp_init_var(&v_a, a);
    exp_init_var(&v_b, b);

    exp_const_t v_1;
    exp_init_const(&v_1, 42);

    exp_add_t add;
    exp_init_add(&add, (exp_t *)&v_a, (exp_t *)&v_1);

    exp_add_t add2;
    exp_init_add(&add2, (exp_t *)&v_b, (exp_t *)&add);
    add2.vtab->print(&add2, stdout);
}