package ast;

import ast.expressions.ConstExpression;
import ast.expressions.Expression;
import ast.expressions.binary.AddExpression;
import ast.expressions.binary.AndExpression;
import ast.expressions.binary.DivExpression;
import ast.expressions.binary.EqualsExpression;
import ast.expressions.binary.MulExpression;
import ast.expressions.binary.OrExpression;
import ast.expressions.binary.SubExpression;
import ast.expressions.binary.XorExpression;
import ast.expressions.unary.NegExpression;
import ast.expressions.unary.NotExpression;

/**
 * Factory class used to get instances of all the different expressions
 */
public class ASTFactory {

    public static Expression createConstExpression(int number) {
        return new ConstExpression(number);
    }

    public static Expression createAddExpression(Expression lhs, Expression rhs) {
        return new AddExpression(lhs, rhs);
    }

    public static Expression createSubExpression(Expression lhs, Expression rhs) {
        return new SubExpression(lhs, rhs);
    }

    public static Expression createMulExpression(Expression lhs, Expression rhs) {
        return new MulExpression(lhs, rhs);
    }

    public static Expression createDivExpression(Expression lhs, Expression rhs) {
        return new DivExpression(lhs, rhs);
    }

    public static Expression createNegExpression(Expression operand) {
        return new NegExpression(operand);
    }

    public static Expression createEqualsExpression(Expression lhs, Expression rhs) {
        return new EqualsExpression(lhs, rhs);
    }

    public static Expression createAndExpression(Expression lhs, Expression rhs) {
        return new AndExpression(lhs, rhs);
    }

    public static Expression createOrExpression(Expression lhs, Expression rhs) {
        return new OrExpression(lhs, rhs);
    }

    public static Expression createXorExpression(Expression lhs, Expression rhs) {
        return new XorExpression(lhs, rhs);
    }

    public static Expression createNotExpression(Expression operand) {
        return new NotExpression(operand);
    }
}
