package ast.expressions.binary;

import ast.expressions.Expression;
import ast.expressions.Operator;

/**
 * This abstract class represents a binary expression
 * 
 * lhs = left operand, rhs = right operand
 */
public abstract class BinaryExpression extends Expression {

    private final Expression lhs, rhs;
    private final Operator operator;

    public BinaryExpression(Expression lhs, Expression rhs, Operator operator) {
        this.lhs = lhs;
        this.rhs = rhs;
        this.operator = operator;
    }

    public Expression getLhs() {
        return lhs;
    }

    public Expression getRhs() {
        return rhs;
    }

    public Operator getOperator() {
        return operator;
    }

    /**
     * This method should return the full expression represented as abstract syntax.
     * Note, that the method can be used by all subclasses.
     * 
     * Tip: use the toString() method of the enum Operator
     */
    @Override
    public String toString() {
        return "Binary_%s[%s, %s]".formatted(this.operator, this.lhs, this.rhs);
    }

}
