package ast.expressions;

/**
 * This abstrac class represents the base of any expression and what methods should be implemented.
 * For most methods, there is a default implementation, but the eval() and toString() functions
 * have to be implemented by every subclass separately.
 */
public abstract class Expression {

    /**
     * This method should evaluate the expression and return the evaluated value as
     * an int.
     * 
     * @return Evaluated value of the expression
     */
    public abstract int eval();

    @Override
    public abstract String toString();

    /**
     * This method should output the abstract syntax of the full expression to
     * the console.
     * Note, that this method can be used by all subclasses.
     */
    public void print() {
        System.out.println(this.toString());
    }

    /**
     * Returns true, if both expressions evaluate to the same value
     */
    @Override
    public boolean equals(Object o) {
        if (o instanceof Expression) {
            Expression exp = (Expression) o;
            return exp.eval() == this.eval();
        }
        return false;
    }

}
