package parser;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

import java.util.List;

import org.junit.Test;

import ast.expressions.Expression;
import lexer.Lexer;
import lexer.Token;
import parser.Parser.ParsingException;

public class SimpleTests {
    
    private void testParser(String concrete, String abstr) throws Exception {
        Lexer lexer = new Lexer();
        Parser parser = new Parser();

        List<Token> tokens = lexer.lex(concrete);
        Expression exp = parser.parse(tokens);
        assertEquals(abstr, exp.toString());
    }

    @Test
    public void testParserFullTree() throws Exception {
        String concrete = "(3 * (((-42) + 3) - 5)) / (2 + ((1 || (((!0) && 0) ^ 1)) && (8 == 1)))";
        String abstr = "Binary_/[Binary_*[Const_3, Binary_-[Binary_+[Unary_~[Const_42], Const_3], Const_5]], Binary_+[Const_2, Binary_&&[Binary_||[Const_1, Binary_^[Binary_&&[Unary_![Const_0], Const_0], Const_1]], Binary_==[Const_8, Const_1]]]]";
    
        testParser(concrete, abstr);
    }

    @Test
    public void testParserAssociativity() throws Exception {
        String concrete = "2 * 3 + 4";
        String abstr = "Binary_+[Binary_*[Const_2, Const_3], Const_4]";
    
        testParser(concrete, abstr);
    }

    @Test
    public void testEmptyExpression() {
        String concrete = "";
    
        assertThrows(ParsingException.class, () -> testParser(concrete, concrete));
    }

    @Test
    public void testEmptyExpression2() {
        String concrete = "()";
    
        assertThrows(ParsingException.class, () -> testParser(concrete, concrete));
    }

    @Test
    public void testTooLongExpression() {
        String concrete = "5 5";
    
        assertThrows(ParsingException.class, () -> testParser(concrete, concrete));
    }
}
